/**
 * Copyright (c) 2010-2012 EBM WebSourcing, 2013-2018 Linagora
 *
 * This program/library is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License as published by
 * the Free Software Foundation, either version 2.1 of the License, or (at your
 * option) any later version.
 *
 * This program/library is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License
 * for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with this program/library; If not, see http://www.gnu.org/licenses/
 * for the GNU Lesser General Public License version 2.1.
 */
package org.ow2.petals.development.enforcer.rules;

import java.io.File;
import java.io.IOException;

import org.apache.maven.enforcer.rule.api.EnforcerRule;
import org.apache.maven.enforcer.rule.api.EnforcerRuleException;
import org.apache.maven.enforcer.rule.api.EnforcerRuleHelper;
import org.apache.maven.plugin.logging.Log;
import org.apache.maven.project.MavenProject;
import org.eclipse.jgit.lib.Repository;
import org.eclipse.jgit.lib.RepositoryBuilder;
import org.tmatesoft.svn.core.SVNException;
import org.tmatesoft.svn.core.wc.SVNClientManager;
import org.tmatesoft.svn.core.wc.SVNInfo;
import org.tmatesoft.svn.core.wc.SVNRevision;

/**
 * A base implementation for Petals enforcer rules.
 */
abstract class AbstractRule implements EnforcerRule {

    private final SVNClientManager svnClientManager = SVNClientManager.newInstance();


    /**
     * The method to implement the rule.
     * @param helper
     * @throws EnforcerRuleException
     */
    protected abstract void doExecute( EnforcerRuleHelper helper ) throws EnforcerRuleException;



    /**
     * Finds the SCM URL in a project by checking the SCM configuration.
     * <p>
     * SVN and Git are supported.
     * </p>
     *
     * @return the current SCM URL as Maven format (starting with "scm:svn:" or "scm:git:") or null if not found
     */
    protected String findScmUrlInProject( MavenProject mavenProject, EnforcerRuleHelper helper) {

        String result = null;
        final File baseDir = mavenProject.getBasedir();
        boolean tryGit = false;

        // Try SVN first
        try {
            final SVNInfo svnInfo = this.svnClientManager.getWCClient().doInfo( baseDir, SVNRevision.WORKING );
            if( svnInfo != null ) {
                if( svnInfo.getURL() != null ) {
                    result = "scm:svn:" + svnInfo.getURL().toString();
                } else {
                    helper.getLog().info( "The project uses SVN but no SVN URL was found. Rule is ignored." );
                }
            }

        } catch( final SVNException e ) {
            helper.getLog().info( "Error while retrieving SCM information for SVN (" + e.getMessage() + "). Trying with Git." );
            tryGit = true;
        }

        // Otherwise, try Git
        if( tryGit ) {
            Repository repo = null;
            try {
                RepositoryBuilder repoBuilder = new RepositoryBuilder().findGitDir( baseDir );
                if( repoBuilder.getGitDir() != null ) {
                    repo = repoBuilder.build();
                    result = repo.getConfig().getString( "remote", "origin", "url" );

                    if( result != null && result.trim().length() > 0 ) {
                        result = "scm:git:" + result;
                        helper.getLog().info( "An URL was found for Git. " );
                    } else {
                        helper.getLog().info( "The project uses Git but no remote URL was found. Rule is ignored." );
                    }
                }

            } catch( IOException e ) {
                helper.getLog().info( "Error while retrieving SCM information for Git (" + e.getMessage() + "). Rule is ignored." );

            } finally {
                if( repo != null )
                    repo.close();
            }
        }

        return result;
    }

    @Override
    public final void execute( EnforcerRuleHelper helper ) throws EnforcerRuleException {
        final Log log = helper.getLog();
        log.info( String.format( "Checking enforcer rule %s.", this.getClass().getSimpleName()));
        doExecute(helper);
    }

    @Override
    public String getCacheId() {
        // N/A because not cacheable.
        return "";
    }

    @Override
    public boolean isCacheable() {
        return false;
    }

    @Override
    public boolean isResultValid( EnforcerRule rule ) {
        return false;
    }
}
